import { Box } from "@mui/material";
import { http } from "msw";
import { IOCTable } from "../../../../components/IOC/IOCTable";
import iocs from "../../../../mocks/fixtures/PagedIOCResponse.json";
import { RouterHarness } from "../../../../mocks/AppHarness";
import {
  hideStorybookControls,
  paginationNoResults
} from "../../../utils/common-args";

export default {
  title: "IOC/IOCTable",
  argTypes: {
    rowType: {
      options: ["explore", "host"],
      control: { type: "radio" }
    },
    iocs: hideStorybookControls,
    pagination: hideStorybookControls,
    onPage: hideStorybookControls
  }
};

const Template = (args) => {
  return (
    <RouterHarness>
      <Box height="90vh">
        <IOCTable {...args} />
      </Box>
    </RouterHarness>
  );
};

export const Empty = (args) => <Template {...args} />;

Empty.args = {
  rowType: "explore",
  loading: false,
  iocs: [],
  pagination: paginationNoResults,
  onPage: () => {}
};

export const EmptyLoading = (args) => <Template {...args} />;

EmptyLoading.args = {
  ...Empty.args,
  loading: true
};

export const BeforeAsync = (args) => <Template {...args} />;

BeforeAsync.args = {
  ...Empty.args,
  pagination: { ...paginationNoResults, totalCount: iocs.totalCount },
  iocs: iocs.iocList
};
BeforeAsync.argTypes = {
  loading: hideStorybookControls
};
BeforeAsync.parameters = {
  msw: {
    handlers: [
      http.get("*/iocs/*", (req, res, ctx) => res(ctx.delay("infinite"))),
      http.get("*/iocs/:ioc_id/status", (req, res, ctx) => {
        res(ctx.delay("infinite"));
      })
    ]
  }
};

export const AfterAsync = (args) => <Template {...args} />;
AfterAsync.args = { ...BeforeAsync.args };
AfterAsync.argTypes = { ...BeforeAsync.argTypes };
