import { useContext, useEffect, useMemo, useCallback, useState } from "react";
import { string } from "prop-types";
import {
  SimpleAccordion,
  useAPIMethod,
  usePagination
} from "@ess-ics/ce-ui-common";
import { Alert, Typography } from "@mui/material";
import { getErrorMessage } from "../../../components/common/Helper";
import { apiContext } from "../../../api/DeployApi";
import { JobTable } from "../../../components/Job";
import { ROWS_PER_PAGE } from "../../../constants";

const propTypes = {
  hostId: string.isRequired
};

export const HostJobsSection = ({ hostId }) => {
  const client = useContext(apiContext);

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: ROWS_PER_PAGE,
    initLimit: ROWS_PER_PAGE[0],
    initPage: 0
  });
  const [expanded, setExpanded] = useState(false);

  const {
    value: hostLog,
    wrapper: getHostLog,
    dataReady,
    error,
    loading,
    abort: abortGetHostLog
  } = useAPIMethod({
    fcn: client.apis.Jobs.listJobs,
    call: false,
    params: useMemo(
      () => ({ host_id: hostId, ...pagination }),
      [hostId, pagination]
    )
  });

  const onPage = useCallback(
    (params) => {
      setPagination(params);
      abortGetHostLog();
    },
    [setPagination, abortGetHostLog]
  );

  useEffect(() => {
    setPagination({ totalCount: hostLog?.totalCount ?? 0 });
  }, [setPagination, hostLog]);

  useEffect(() => {
    if (expanded) {
      getHostLog();
    }
  }, [expanded, pagination, getHostLog]);

  useEffect(() => {
    return () => {
      abortGetHostLog();
    };
  }, [abortGetHostLog]);

  return (
    <SimpleAccordion
      expanded={expanded}
      summary={
        <Typography
          variant="h3"
          component="h2"
        >
          Job log
        </Typography>
      }
      onChange={(_, expanded) => setExpanded(expanded)}
    >
      {error ? <Alert severity="error">{getErrorMessage(error)}</Alert> : null}
      {hostLog ? (
        <JobTable
          jobs={!error && hostLog ? hostLog?.jobs : null}
          customColumns={[
            { field: "status" },
            { field: "job" },
            { field: "user" }
          ]}
          loading={loading || !dataReady}
          pagination={pagination}
          onPage={onPage}
        />
      ) : null}
    </SimpleAccordion>
  );
};

HostJobsSection.propTypes = propTypes;
