import { useContext, useCallback, useEffect } from "react";
import { Box } from "@mui/material";
import {
  RootPaper,
  useAPIMethod,
  usePagination,
  usePolling
} from "@ess-ics/ce-ui-common";
import { initRequestParams } from "../../components/common/Helper";
import { JobTable } from "../../components/Job/JobTable";
import { apiContext } from "../../api/DeployApi";
import { ROWS_PER_PAGE } from "../../constants";

export function JobListView() {
  const client = useContext(apiContext);

  const {
    value: jobs,
    wrapper: getJobs,
    loading,
    dataReady,
    abort
  } = useAPIMethod({
    fcn: client.apis.Jobs.listJobs,
    call: false
  });

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: ROWS_PER_PAGE,
    initLimit: ROWS_PER_PAGE[0],
    initPage: 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: jobs?.totalCount ?? 0 });
  }, [setPagination, jobs?.totalCount]);

  const callGetOperations = useCallback(() => {
    let requestParams = initRequestParams(pagination);
    getJobs(requestParams);
  }, [getJobs, pagination]);

  // Request new search results whenever search or pagination changes
  useEffect(() => {
    callGetOperations();

    return () => {
      abort();
    };
  }, [abort, callGetOperations, pagination]);

  usePolling(callGetOperations, loading, 30000, abort, false);

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
    abort();
  };

  return (
    <RootPaper>
      <Box paddingY={2}>
        <JobTable
          jobs={jobs?.jobs}
          pagination={pagination}
          onPage={onPage}
          loading={loading || !dataReady}
        />
      </Box>
    </RootPaper>
  );
}
